#!/usr/bin/python3
import json
import os
from typing import Tuple

import github
import re
import time
import datetime
import traceback

import github.GithubException
import dateutil.parser
import requests
from psycopg_pool import ConnectionPool

TOPICS = [
    'android-app',
    'android',
    "android app",
    'android-tv',
    'android-application',
    'android-apps',
    'android-developers',
    'android-applications',
    'android-tv-apps',
    'android-tv-application',
    # 'android-library',
    "android watch",
    "android-watch"
]

# criteria:
# - last commit after Jan 1, 2022
# - star > 50
# - number of commits > 200

GITHUB_TOKEN = "##REMOVED##"

g = github.Github(login_or_token=GITHUB_TOKEN)

repos = list()
failed = list()
raw_data = open("raw_data.json", "a")

pool = ConnectionPool("dbname=fdroid_app user=postgres")


def sql_exec(sql, params=None):
    with pool.connection() as conn:
        with conn.cursor() as cur:
            cur.execute(sql, params=params)


def check_exists(repo: str):
    with pool.connection() as conn:
        with conn.cursor() as cur:
            cur.execute("SELECT COUNT(*) from github_app where fullname = %s", (repo,))
            result = cur.fetchone()
            return int(result[0]) > 0


def iterative(topic, from_date: datetime.datetime) -> Tuple[datetime.datetime, int]:
    print(f"find {topic} with date {str(from_date)}")
    last_date = None
    result = None
    try:
        result = g.search_repositories(query=f'{topic} stars:>50 pushed:>={from_date.strftime("%Y-%m-%d")} sort:updated-asc language:java language:kotlin', )
        print(result.totalCount)
        i = 0
        for repo in result:
            i += 1
            time.sleep(0.05)
            last_date = repo.pushed_at
            print(f"[*] found {repo.full_name}")
            if repo.full_name in repos or repo.full_name in failed:
                continue
            if check_exists(repo.full_name):
                continue
            if crawl(repo, topic):
                repos.append(repo.full_name)
                print(repo.full_name)
            else:
                failed.append(repo.full_name)
            time.sleep(0.05)
        return last_date, result.totalCount
    except Exception as e:
        traceback.print_exc()
        pass
    print(repos)
    return last_date, result.totalCount


def crawl(ghrepo: github.Repository, found_by):
    raw_data.write(str(ghrepo.raw_data))
    raw_data.write("\n")
    raw_data.flush()
    print("crawl", ghrepo.full_name)
    time.sleep(0.05)
    commit_number = ghrepo.get_commits().totalCount
    if commit_number <= 200:
        print("fail", ghrepo.full_name, "less than 200 commits")
        return False
    # default_branch = ghrepo.get_branch(ghrepo.default_branch)
    # time.sleep(0.05)
    # date = str(default_branch.commit.commit.committer.date)
    # last_commit_date = datetime.datetime.fromisoformat(date)
    # if last_commit_date <= DATE_BEFORE:
    #     print("fail", repo, "last commit before requirement")
    #     return False
    topics = ",".join(ghrepo.topics)
    sql_exec("INSERT INTO github_app VALUES (%s, %s, %s, %s, %s, %s, %s, %s)", (ghrepo.full_name, ghrepo.stargazers_count, topics, commit_number, str(ghrepo.pushed_at), found_by, ghrepo.language, ghrepo.description))
    print("done", ghrepo)
    return True


if __name__ == '__main__':
    lst = os.listdir()
    lst2 = list(filter(lambda x: x.startswith('skipped-'), lst))
    lst2.sort()
    lst3 = list(filter(lambda x: x.startswith('crawled-'), lst))
    lst3.sort()
    print(f"load old data from {lst2[-1]}, {lst3[-1]}")
    with open(lst2[-1], "r") as f:
        repos = json.load(f)
    with open(lst3[-1], "r") as f:
        failed = json.load(f)
    print("reload lists")
    for topic in TOPICS:
        # if topic != "android-application":
        #     continue
        next_date = datetime.datetime(2022, 1, 1)
        while True:
            next_date, count = iterative(topic, next_date)
            if next_date is None:
                break
            if count <= 1000:
                break
        with open(f"crawled-{int(time.time())}.json", "w") as f:
            f.write(json.dumps(repos))
        with open(f"skipped-{int(time.time())}.json", "w") as f:
            f.write(json.dumps(failed))
    raw_data.flush()
    raw_data.close()
    with open(f"crawled-{int(time.time())}.json", "w") as f:
        f.write(json.dumps(repos))
    with open(f"skipped-{int(time.time())}.json", "w") as f:
        f.write(json.dumps(failed))

import signal


def signal_handler(signal, frame):
    with open(f"crawled-{int(time.time())}.json", "w") as f:
        f.write(json.dumps(repos))
    with open(f"skipped-{int(time.time())}.json", "w") as f:
        f.write(json.dumps(failed))
    exit(1)


signal.signal(signal.SIGINT, signal_handler)
